/*
 * Copyright (C) 2012 Marcus Overhagen <marcus@overhagen.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "spi.h"
#include "util.h"


#define DEBUG_SIGNAL 0


void
spi_init(void)
{
#if defined(__AVR_ATmega103__)
	//Fsck = Fcpu/4
	DDRB |= _BV(PB0) | _BV(PB1) | _BV(PB2);
	SPCR = _BV(SPE) | _BV(MSTR);
#elif defined(__AVR_ATmega1284P__)
	//Fsck = Fcpu/2
	DDRB |= _BV(PB4) | _BV(PB5) | _BV(PB7);
	SPCR = _BV(SPE) | _BV(MSTR);
	SPSR |= _BV(SPI2X);
#else
	#error unsupported
#endif

#if DEBUG_SIGNAL
	DDRD |= _BV(PD7);
#endif
}



/* Write RGB 8 bit data to LPD8806 chips using hardware SPI.
 * Does conversion from 8 bit to 7 bit and the protocol stuff.
 * Must be called for length of 3 bytes (1 LED) or more,
 * doesn't work for 1 byte transfers.
 */
void
spi_write_lpd8806(const uint8_t *data, uint16_t length)
{
	const uint8_t *endofdata;
	uint8_t nextbyte;

#if DEBUG_SIGNAL
	PORTD |= _BV(PD7);
#endif

	endofdata = data + length;

	SPDR = *data++ >> 1 | 0x80;
	nextbyte = *data++ >> 1 | 0x80;
	for (;;) {
		loop_until_bit_is_set(SPSR, SPIF);
		SPDR = nextbyte;
		if (data == endofdata)
			break;
		nextbyte = *data++ >> 1 | 0x80;
	}
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = 0x00;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = 0x00;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = 0x00;
	loop_until_bit_is_set(SPSR, SPIF);

#if DEBUG_SIGNAL
	PORTD &= (uint8_t)~_BV(PD7);
#endif
}
